#**********************************************************************************************************#
# Méthode de Monte-Carlo pour le calcul de la valeur moyenne et de l'incertitude-type de la concentration  #
# molaire Co de la solution mère So                                                                        #
# Programme publié par le GRIESP en 2019 - Modifications mineures apportées par Marie-Anne DEJOAN          #
#**********************************************************************************************************#

from scipy import *
import numpy as np
from matplotlib import pyplot as plt

# Renvoie une valeur aléatoire de la variable L[0] d'incertitude-type L[1]
def Alea(L):
    tirage=np.random.normal()   # Tirage entre -infini et +infini (loi normale)
    return L[0]+L[1]*tirage

# Entrées
m=[584.e-3,1.e-3] # [valeur,incertitude-type] de chaque entrée
M=[58.44,0.01]
Vo=[0.50000,0.00025]

# Calcul de Co
Co=m[0]/(M[0]*Vo[0])

# Méthode de Monte Carlo pour trouver l'incertitude sur Co sans formule de propagation des incertitudes
LCo=[] # Initialisation de la liste LCo qui contiendra les valeurs successives de Co calculées par itération
iteration=100000

for i in range(iteration):
    AleaCo=Alea(m)/(Alea(M)*Alea(Vo))
    LCo.append(AleaCo) # Ajout de la dernière valeur calculée à la liste LCo

MoyCo = mean(LCo)      # Calcul de la valeur moyenne de Co (à partir des éléments qui composent la liste LCo)
uCo=std(LCo, ddof = 1) # Ecart-type expérimental = incertitude-type de la grandeur Co

# Affichage
print('Calcul de Co :',Co,'mol/L')
print('Moyenne des Co :',MoyCo,'mol/L')
print('Incertitude-type u(Co) :',uCo,'mol/L')
plt.hist(LCo, range=(min(LCo),max(LCo)), bins = 50, color = 'blue', edgecolor = 'black')
plt.xlabel('Co (mol/L)')
plt.ylabel('Effectif')
plt.title('Pour 100 000 iterations')
plt.show()

