#***********************************************************************************************************#
# Méthode de Monte-Carlo pour le calcul de la concentration massique et l'incertitude-type en NaCl du sérum #
# physiologique commerciale                                                                                 #
# Programme publié par le GRIESP en 2019 - Modifications mineures apportées par Marie-Anne DEJOAN           #
#***********************************************************************************************************#

from scipy import *
from scipy.stats import linregress
import numpy as np
from matplotlib import pyplot as plt

# Renvoie une valeur aléatoire de la variable L[0] d'incertitude-type L[1]
def Alea(L):
    tirage=np.random.normal()   # Tirage entre -infini et +infini (loi normale)
    return L[0]+L[1]*tirage

# Entrées
m=[584.e-3,1.e-3]     # [valeur,incertitude-type] de chaque entrée
M=[58.44,0.01]
Vo=[0.50000,0.00025]
Vf=[50e-3,0.06e-3]    # Volume de chaque solution fille (en L)

Vmere=[10e-3,15e-3,20e-3,30e-3,40e-3]     # Volume de solution mère prélevé pour préparer chaque solution fille (en L)
u_pipettes=[4e-5,5e-5,6e-5,7.2e-5,8.5e-5] # Incertitude associée à chaque volume de solution mère prélevé (en L)

#---------------------------------Partie du script à compléter par les élèves--------------------------------------
cond=[...,...,...,...,...]                # Conductivité mesurée pour chaque solution fille (en mS/cm)
InrelCond=0.02                            # Incertitude relative sur la mesure de la conductivité égale à 2%

Report=[...,...]       # Conductivité et incertitude de la solution commerciale diluée de sérum physiologique

Vserum=[5e-3,0.1e-3]   # Volume de sérum physiologique contenu dans une dosette et incertitude (estimée) (en L)
Vfiole=[100e-3,0.1e-3] # Volume et incertitude de la fiole jaugée utilisée pour diluer le serum physilogique (en L)
#------------------------------------------------------------------------------------------------------------------

# Préparation des listes avec incertitudes
sigma=[]
for k in range(len(cond)):
    sigma.append([cond[k],cond[k]*InrelCond])

Vm=[]
for k in range(len(Vmere)):
    Vm.append([Vmere[k],u_pipettes[k]])

# Méthode de Monte Carlo pour déterminer la concentration massique en NaCl du sérum physiologique
LCmass=[]
iteration=100000

for i in range(iteration):
    AleaM=Alea(M)
    AleaCm=Alea(m)/(AleaM*Alea(Vo))
    AleaCf=[]
    AleaSigma=[]
    for k in range(len(Vmere)):
        AleaCf.append(AleaCm*Alea(Vm[k])/Alea(Vf))
        AleaSigma.append(Alea(sigma[k]))
    AleaReport=Alea(Report)
    reg=linregress(AleaCf,AleaSigma)
    coef=reg[0]
    ord=reg[1]
    C_dilue=(AleaReport-ord)/coef
    LCmass.append(AleaM*C_dilue*Alea(Vfiole)/Alea(Vserum))

MoyCmass=mean(LCmass)
uCmass=std(LCmass, ddof = 1)

print('Concentration massique en NaCl du sérum :', MoyCmass, 'g/L')
print('Incertitude-type :',uCmass,'g/L')

plt.hist(LCmass, range = (min(LCmass),max(LCmass)), bins = 50, color = 'blue', edgecolor = 'black')
plt.xlabel('Concentration massique en NaCl du sérum (en g/L)')
plt.ylabel('Effectif')
plt.title('Pour 100 000 iterations')
plt.show()

